/*
   Copyright 2000, 2001, 2002, 2003 Laurent Wacrenier

   This file is part of PLL lmtpd.

   PLL lmtpd is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   PLL lmtpd is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with PLL lmtpd; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

#include "config.h"

static char const rcsid[] UNUSED =
"$Id: error.c,v 1.19 2004/10/26 09:59:20 lwa Exp $";



#include <errno.h>
#include <syslog.h>

#include <stdio.h>
#include <stdarg.h>

#include "lmtp.h"

char *seterr(struct fstate *state, int code, char *ecode, char *fmt, ...) {
  if (state == NULL) {
    if (code == 0) {
      return "";
    } else if (code >= 200 && code <= 299 ) {
      return "200 2.0.0";
    } else if (code >= 400 && code <= 499) {
      return "400 4.0.0";
    } else {
      return "500 5.0.0";
    }
  } else {
    char *error=state->error;
    char *end;
    va_list ap;
    
    if (state->fatal)    /* dont change error message in fatal condition */
      return state->error;
    
    switch(code) {
    case 421: case 221:
      state->fatal=1;
      break;
    case 0:              /* reset error message */
      state->error[0]=0;
      return state->error;
    }
    
    sprintf(error, "%03d %5.5s ", code, ecode);
    error+=10;
    
    va_start(ap, fmt);
    end = error + vsnprintf(error, ERRMSGMAX-14, fmt, ap);
    va_end(ap);


    while(*end && *end != '\n' && *end != '\r')
      end++;

    if (state->reply_id) {
      end += snprintf(end, ERRMSGMAX - (error - end) - 1, 
		      "; id=%lu.%03lu", (long int)state->pid, state->counter);
      if (end > error + ERRMSGMAX-4)
	end = error + ERRMSGMAX-4;
    }


    *end++='\r';
    *end++='\n';
    *end++='\0';
    
    
    return state->error;
  }
}
