# -*- perl -*-
# $Id: lmtpd-filter.pl,v 1.1 2003/02/07 16:43:18 lwa Exp $

use strict;
use Safe;
use Symbol;

package Mail;
use overload 
#    '${}' => \&ov_size,
    '@{}' => sub { shift()->{headers} },
    '0+'  => sub { shift()->{size} },
#    '""'  => undef,
    '&{}' => \&run;
#    fallback => 1;
    ;

sub run {
    my $mail = shift;
    my $h = $mail->hheaders();
    return sub { $h->{lc shift} };
}

sub hheaders {
    my $mail = shift;
    unless (exists $mail->{hheaders}) {
	my %h;
	for my $h (@{$mail->{headers}}) {
	    my ($h1, $h2) = split(/\s*:\s*/, $h, 2); # keep taint
	    $h{lc $h1} .= $h2;
	}
	$mail->{hheaders} = \%h;
    }
    return $mail->{hheaders};
}

sub is_tainted (@) {
    return ! eval {
	join('',@_), kill 0;
	1;
    };
}

sub bounce ($$) {
    my $mail = shift;
#    print STDERR "tainted bounce\n" if &is_tainted(@_);
    $mail->{bounce} = join("", @_);
    return $mail;
}

sub vacation ($@) {
    my $mail = shift;
    if (@_) {
	$mail->{vacation} = join("\n", @_);
    } else {
	delete $mail->{vacation};
    }
    return $mail;
}

sub forward ($$) {
    my $mail = shift;
    push @{$mail->{forward}}, shift;
    return $mail;
}

sub deliver ($$) {
    my $mail = shift;
    push @{$mail->{deliver}}, shift()."";
    return $mail;
}

sub trash ($$) {
    my $mail = shift;
    $mail->deliver("Trash");
    $mail->delete($mail->{vacation});
    $mail->bounce(@_) if @_;
    return $mail;
}

sub match ($$) {
    my $mail = shift;
    my $expr = shift; 
    print STDERR "EXPR = $expr\n";
    for my $h (@{$mail->{headers}}) {
	return 1 if $h =~ $expr;
    }
    return 0;
}

package main;

sub filter ($$$$) {
    my 	$safe = new Safe; # "Root";
    my $script;
    $safe->share_from('Mail',
		      [qw/&bounce &vacation &trash &deliver &forward
		       &match
		       /])
	or warn $@;
    $safe->permit_only qw/:base_core gv :base_io :base_loop :base_orig/;
#    $safe->permit_only qw/:base_core gv :base_io :base_loop/;

    my ($mbox, $name, $fd, $mail) = @_;
    print STDERR "PERL: $mbox, $name, $fd\n";
    {
	open F, "<&=$fd" or warn "open: $!" ;
	local($/) = undef;
	$script = <F>;
	close F;
    }

#    $SIG{__DIE__} = sub {};
    $script = $1 if $script =~ /^(.*)$/s;   # untaint
#    print STDERR "$script \n";
    $_ = $mail;
    bless $_, "Mail";
    my $ret = $safe->reval($script);
#   my $ret = eval $script;
    if (! defined $ret && defined $@) {
	$ret = $@;
#	$ret =~ s/ at \(eval \d+\) line (\d+)$/ at line $1/s;
	chomp($ret);
	return $ret;
    }

    return $ret;
};
