/*
   Copyright 2000, 2001, 2002, 2003 Laurent Wacrenier

   This file is part of PLL lmtpd.

   PLL lmtpd is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   PLL lmtpd is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with PLL lmtpd; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

#include "config.h"

static char const rcsid[] UNUSED =
"$Id: mailaddr.c,v 1.16 2005/04/26 09:42:56 lwa Exp $";



#ifdef DMALLOC
#include <dmalloc.h>
#endif

#include <string.h>
#include <stdlib.h>
#include <ctype.h>
#include <stdio.h>

static char *check_ipv4_litteral(char *m) {
  unsigned long l;
  char *end;
  l = strtoul(m, &end, 10);
  if (l >= 224 || l == 0 || l == 127 ||
      end > m + 3 || *end != '.') /* a bit more strict */
    return NULL;
  m = end + 1;
  l = strtoul(m, &end, 10);
  if (l > 255 || (*m == '0' && l != 0) || end > m + 3 || *end != '.')
    return NULL;
  m = end + 1;
  l = strtoul(m, &end, 10);
  if (l > 255 || (*m == '0' && l != 0) || end > m + 3 || *end != '.')
    return NULL;
  m = end + 1;
  l = strtoul(m, &end, 10);
  if (l > 255 || (*m == '0' && l != 0) || end > m + 3)
    return NULL;
  return end;
}


/* TODO: code this :

      IPv6-addr = IPv6-full / IPv6-comp / IPv6v4-full / IPv6v4-comp
      IPv6-hex  = 1*4HEXDIG
      IPv6-full = IPv6-hex 7(":" IPv6-hex)
      IPv6-comp = [IPv6-hex *5(":" IPv6-hex)] "::" [IPv6-hex *5(":"
							       IPv6-hex)]
      ; The "::" represents at least 2 16-bit groups of zeros
      ; No more than 6 groups in addition to the "::" may be
      ; present
      IPv6v4-full = IPv6-hex 5(":" IPv6-hex) ":" IPv4-address-literal
      IPv6v4-comp = [IPv6-hex *3(":" IPv6-hex)] "::"
      [IPv6-hex *3(":" IPv6-hex) ":"] IPv4-address-literal
      ; The "::" represents at least 2 16-bit groups of zeros
      ; No more than 4 groups in addition to the "::" and
      ; IPv4-address-literal may be present
*/

static char *check_ipv6_litteral(char *m) {
      size_t spaned = strspn(m, "0123456789aAbBcCdDeEfF:.");
      m+= spaned;
      return m;
}

char *unquote_addr821(char *m, int allow_no_domain, char **rest) {
  int cont=1;
  char *ret=strdup(m);
  char *d=ret;
  int candot;

  *rest=NULL;
  if (ret==NULL)
    return NULL;
  if (*m=='"') {                 /* quoted string */
    m++;
    while(*m && cont>0) {
      if (*m=='\\') {
	m++;
	if (*m==0 || !isascii((int)*m)) {
	  cont = -1;
	  break; /* error */
	}
	*d++=*m++;
      } else if (*m=='"') {
	m++; cont=0;
	break;
      } else {
	if (!isascii(*m)) {
	  cont=-1;
	  break;
	}
	*d++=*m++;
      }
    }
  } else {                         /* dot string */
    candot=0;
    while(*m && cont>0) {
      switch(*m) {
      case '<': case '>': case '(': case ')':
      case '[': case ']': case '"': case ':':
      case ',': case ';':
	cont=allow_no_domain ? 0 : -1;
	break; /* error */
      case '\\':
	m++;
	if (*m==0 || !isascii((int)*m)) {
	  cont=-1;
	  break; /* error */
	}
	candot=1;
	*d++=*m++;
	break;
      case '@':
	if (candot<0)
	  cont=-1;
	else
	  cont=0;
	break; /* ok */
      case '.': /*  faire: viter les doubles */
	if (candot<=0) {
	  cont=-1;
	  break;
	}
	*d++=*m++;
	candot=-1;
	break;
      default:
	if (iscntrl((int)*m) || *m==' ' || !isascii((int)*m)) {
	  cont=-1;
	  break;
	}
	candot=1;
	*d++=*m++;
      }
    }
  }

  if (cont<0 || (*m!='@' && !allow_no_domain) || d==ret) {
    free(ret);
    *rest=m;
    return NULL;
  }

  if (*m != '@') { /* allow_no_domain should be true here  */
    *d = 0;
    *rest = m;
    return ret;
  }

  *d++=*m++;

  if (*m == '[') { 
    char *end;
    if (strncasecmp(m+1, "ipv6:", sizeof("ipv6:")-1) == 0) {
      end = check_ipv6_litteral(m + sizeof("[ipv6:") - 1);
    } else {
      end = check_ipv4_litteral(m + 1);
    }
    if (end == NULL || *end != ']') {
      free(ret);
      return NULL;
    }
    while(m <= end)
      *d++ = *m++;
  } else { /* domain */

    candot=0;
    /* vrification rudimentaire (les registrars ne suivent pas la RFC 821) */
    while(*m) {
      if (isalnum((int)*m) || *m == '-') {
	candot=1;
	*d++=*m++;
      } else if (*m == '.') {
	if (candot==0)
	break;
	candot=0;
	*d++=*m++;
      } else {
	break;
      }
    }
    if (candot==0) {
      free(ret);
      ret = NULL;
    }
  }

  *rest=m;
  *d=0;
  return ret;

}

int delimit_extension(char *addr, char delimiter,
		       char **recipient, char **ext) {
  char *ext_start;
  char *ext_end;
  size_t addr_len;
  if (addr ==NULL) {
    *recipient = NULL;
    *ext = NULL;
    return 0;
  }
  if (delimiter == 0) {
    *recipient = strdup(addr);
    *ext = NULL;
    return 0;
  }
  addr_len = strlen(addr);
  ext_end = strrchr(addr, '@');
  if (ext_end == NULL)
    ext_end = addr + addr_len;
  ext_start = strchr(addr, delimiter);
  if (ext_start == NULL || ext_start >= ext_end) {
    *recipient = strdup(addr);
    *ext = NULL;
    return 0;
  }
  *ext = malloc(ext_end-ext_start);
  if (*ext == NULL)
    return -1;
  if (ext_end-ext_start -1 > 0)
    memcpy(*ext, ext_start+1, ext_end-ext_start -1);
  (*ext)[ext_end-ext_start -1] = 0;

  *recipient = malloc(addr_len - (ext_end-ext_start) + 1);
  if (*ext == NULL)
    return -1;
  memcpy(*recipient, addr, ext_start - addr);
  memcpy(*recipient + (ext_start - addr), ext_end, addr + addr_len - ext_end);
  (*recipient)[addr_len - (ext_end - ext_start)] = 0;
  return 0;
}

char *quote_addr821(char *m) {
  char *s=m;
  char *at;
  int candot=0;
  int toquote=0;
  int addlen=0;
  char *ret;
  if (m==NULL)
    return NULL;
  at = strrchr(m, '@');
  while(*s) {
    if (s < at) { /* check local part */
      switch(*s) {
      case '\r': case '"': case '\n': case '\\':
	addlen++; /* no break */
      case '<': case '>': case '[': case ']': case '(': case ')':
      case ':': case ';': case ' ':
	candot=1;
	toquote=1;
	break;
      case '@':
	if (s != at || candot==0)
	  toquote=1;
	candot=1;
	break;
      case '.':
	if (candot==0)
	  toquote=1;
	candot=0;
	break;
      default:
	candot=1;
	if (iscntrl((int)*s) || !isascii((int)*s))
	  toquote=1;
	break;
      }
    }
    s++;
  }

  if (!at) at=s;

  if (toquote) {
    char *d;
    d=ret=malloc(s-m+3+addlen);
    if (ret==NULL)
      return NULL;
    s=m;
    *d++='"';
    while(s<at) {
      switch(*s) {
      case '\r': case '"': case '\n': case '\\':
	*d++='\\';  /* no break */
      default:
	*d++=*s++;
	break;
      }
    }
    *d++='"';
    while(*s) {
      *d++=*s++;
    }
    *d='\0';
  } else {
    ret=malloc(s-m+1);
    if (ret==NULL)
      return NULL;
    memcpy(ret, m, s-m+1);
  }
  return ret;
}

#ifdef TEST
#include <stdio.h>

int main(int argc, char **argv) {
  char *s=argv[1];
  char *d=unquote_addr821(s, 1, &s);
  char *q=quote_addr821(d);
  char *r = NULL;
#if 0
    char *e;
    extension(d, '+', &r, &e);
#endif
  printf("addr = '%s'\n", argv[1]);
  printf("rest = '%s'\n", s);
  printf("unquoted = '%s'\n", d);
  printf("requoted = '%s'\n", q);
#if 0
  printf("recipient = '%s'\n", r);
  printf("extension = '%s'\n", e);
#endif
  return 0;
}

#endif
