/*
   Copyright 2000, 2001, 2002, 2003 Laurent Wacrenier

   This file is part of PLL lmtpd.

   PLL lmtpd is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   PLL lmtpd is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with PLL lmtpd; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

#include "config.h"

static char const rcsid[] UNUSED =
"$Id: module.c,v 1.9 2004/10/26 09:59:20 lwa Exp $";

/*
 *  fonctions pour la commande "module" des filtres
 */



#include <stdlib.h>
#include <string.h>
#include "lmtp.h"

void free_mod_cache(struct mod_result *cache, int ncache) {
  int i;
  if (ncache!=0) {
    for (i=0; i<ncache; i++)
      free(cache[i].module);
    free(cache);
  }
}

static int mod_cache_get(struct fstate *state, char *module, long *score) {
  int i;
  if (state->module_cache_size != 0) {
    for(i=0; i<state->module_cache_size; i++) {
      if (strcmp(module, state->module_cache[i].module)==0) {
	*score = state->module_cache[i].result;
	return 0;
      }
    }
  }
  return -1;
}

static int mod_cache_put(struct fstate *state, char *module, long score) {
  struct mod_result *cache;
  char *xmodule=strdup(module);
  int ncache=state->module_cache_size;
  if (xmodule==NULL) {
    return -1;
  }
  cache=realloc(state->module_cache,
		(ncache+1)*sizeof(struct mod_result));
  if (cache==NULL) {
    free(xmodule);
    return -1;
  }
  cache[ncache].module=xmodule;
  cache[ncache].result=score;
  state->module_cache_size++;
  state->module_cache=cache;
  return 0;
}

int cmd_return(struct fstate *state, char *a) {
  if (*a) {
    state->score=readscore(state, &a);
  }
  return FILTER_STOP;
}

int cmd_module(struct fstate *state, char *a) {
  char file[PATH_MAX];
  long score;
  extern struct cmd module_cmdlist[];

  if (state->module_path==NULL)
    return FILTER_CONTINUE;
  if (*a==0 || strchr(a, '/') || *a=='.')
    return FILTER_CONTINUE;
  if (strlen(state->module_path)+strlen(a)>=PATH_MAX-3)
    return FILTER_CONTINUE;
  state->score=0;
  state->tag[0]=0;
  if (mod_cache_get(state, a, &score)==0) {
    state->score=score;
  } else {
    char *charset = state->charset;
    strcpy(file, state->module_path);
    strcat(file, "/");
    strcat(file, a);
    filterfile(state, file, module_cmdlist, NULL, NULL);
    mod_cache_put(state, a, state->score);
    state->tag[0]=0;
    state->charset = charset;
  }
  return FILTER_CONTINUE;
}
