/*
   Copyright 2000, 2001, 2002, 2003 Laurent Wacrenier

   This file is part of PLL lmtpd.

   PLL lmtpd is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   PLL lmtpd is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with PLL lmtpd; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

#include "config.h"

static char const rcsid[] UNUSED =
"$id: mysqllog.c,v 1.5 2001/01/31 15:44:07 lwa Exp $";



#if WITH_MYSQLLOG

#include <syslog.h>
#include <stdlib.h>
#include <signal.h>
#include <mysql.h>
#include <string.h>

#include "lmtp.h"

#ifdef LINEMAX
#undef LINEMAX
#endif

#define MAXROWLEN  255
#define LINEMAX    8192

#if !HAVE_SIG_T
typedef void (*sig_t) (int);
#endif


void mysqlcloselog(struct fstate *state) {
  if (state->mysqllog_opened) {
    mysql_close(&state->mysqllog);
    state->mysqllog_opened=0;
  }
}

static int mysqlopenlog(struct fstate *state) {
  if (state->mysqllog_opened && mysql_ping(&state->mysqllog)!=0) {
    syslog(LOG_ERR, "MySQL LOG ping: %s", mysql_error(&state->mysqllog));
    mysql_close(&state->mysqllog);
    state->mysqllog_opened=0;
  }
  if (state->mysqllog_opened) 
    return 1;
  mysql_init(&state->mysqllog);

  if (mysql_real_connect(&state->mysqllog,
			 state->mysqllog_host,
			 state->mysqllog_user,
			 state->mysqllog_password,
			 state->mysqllog_database,
			 0, NULL, 0)) {
    state->mysqllog_opened=1;
    return 1;
  }
  syslog(LOG_ERR, "MySQL LOG open: %s", mysql_error(&state->mysqllog));
  return 0;
}

static char *qstring(MYSQL *mysql, char *fquery_e, char *s) {
  if (s) {
    char *se;
    int len;
    if ((se=strchr(s, '\r'))!=NULL)
      len = se-s;
    else
      len=strlen(s);

    if (len>MAXROWLEN)
      len=MAXROWLEN;
    *fquery_e++='\'';
    fquery_e+=mysql_real_escape_string(mysql, fquery_e, s, len);
    *fquery_e++='\'';
  } else {
    strcpy(fquery_e, "NULL");
    fquery_e+=sizeof("NULL")-1;
  }
  return fquery_e;
}

#define INT_SPACE (4 * sizeof(long))

static char *qint(MYSQL *mysql, char *fquery_e, long l) {
  char buf[INT_SPACE];
  sprintf(buf, "%ld", l);
  strcpy(fquery_e, buf);
  return fquery_e+strlen(buf);
}

#define QSTRING(x) fquery_e=qstring(mysql, fquery_e, x)
#define QINT(x) fquery_e=qint(mysql, fquery_e, (long)x)

static int doquery(struct fstate *state, char *buf) {
  char fquery[LINEMAX];
  char *fquery_e = fquery;
  MYSQL *mysql = &state->mysqllog;
  char *query = state->mysqllog_query;

  *fquery_e=0;

  while(*query) {
    if (*query != '%') {
      *fquery_e++=*query++;
    } else {
      if (fquery_e > fquery + LINEMAX - MAXROWLEN*2+1) {
	syslog(LOG_ERR, "MySQL LOG query too long");
	return 0;
      }
      switch(*++query) {
      case 'E':	QSTRING(state->error); break;
      case 'm': QSTRING(state->rcpt ? state->rcpt->mbox : NULL); break;
      case 'r': QSTRING(state->rcpt ? state->rcpt->user : NULL); break;
      case 'M': QSTRING(state->messageid); break;
      case 's': QSTRING(state->sender); break;
#if 0
	/* disabled */
      case 'i': QSTRING(*state->map_status ? state->map_status : NULL); break;
#endif
      case 'i': QSTRING(NULL); break;
      case 'S': QINT(state->msgsize); break;
      case 'p': QINT(state->pid); break;
      case 'k': QINT(state->counter); break;
      case 'L': QSTRING(state->lhlo); break;
      case 'P': QSTRING(state->herhostname); break;
      case 'H': QSTRING(state->myhostname); break;
      case 'q': QSTRING(buf); break;
      case 'T': QINT(state->date_start); break;
      case 'A': QSTRING(state->actions && state->actions->action[0] ? 
			state->actions->action : NULL); break;
      case '%': *fquery_e++=*query; break;
      default:
	QSTRING(NULL); break;
      }
      query++;
    }
  }
  return mysql_real_query(mysql, fquery, fquery_e - fquery);
}

int mysqllog(struct fstate *state, char *buf) {
  sig_t s;

  if (state->mysqllog_query == NULL) {
    return 1;
  }
  s=signal(SIGPIPE, SIG_IGN);

  if (mysqlopenlog(state)==0) {
    signal(SIGPIPE, s);
    return 0;
  }

  if (doquery(state, buf) != 0) {
    syslog(LOG_ERR, "MySQL LOG query: %s",  mysql_error(&state->mysqllog));
    signal(SIGPIPE, s);
    return 1;
  }
  signal(SIGPIPE, s);
  return 0;
}

#endif

