/*
   Copyright 2000, 2001, 2002, 2003 Laurent Wacrenier

   This file is part of PLL lmtpd.

   PLL lmtpd is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   PLL lmtpd is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with PLL lmtpd; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

#include "config.h"

#if WITH_ONIGURUMA

static char const rcsid[] UNUSED =
"$Id: re-oniguruma.c,v 1.3 2004/10/27 10:42:08 lwa Exp $";

#include <oniguruma.h>
#include <stddef.h>

#include "re.h"

#define ONI_UTF8   0x10

#define ONI_SYNTAX          0xF

static RegSyntaxType *oni_syn[] = {
#define ONI_RUBY            0x0
  REG_SYNTAX_RUBY,
#define ONI_POSIX_BASIC     0x1
  REG_SYNTAX_POSIX_BASIC,
#define ONI_POSIX_EXTENDED  0x2
  REG_SYNTAX_POSIX_EXTENDED,
#define ONI_EMACS           0x3
  REG_SYNTAX_EMACS,
#define ONI_GREP            0x4
  REG_SYNTAX_GREP,
#define ONI_GNU_REGEX       0x5
  REG_SYNTAX_GNU_REGEX,
#define ONI_JAVA            0x6
  REG_SYNTAX_JAVA,
#define ONI_PERL            0x7
  REG_SYNTAX_RUBY,
};


static int re_compile(RE *re, char *pattern, int icase) {
  int ret;
  RegErrorInfo err_info;
  regex_t *code;
  ret = regex_new(&code, pattern, pattern + strlen(pattern),
		  icase ? REG_OPTION_IGNORECASE : 0 ,
		  re->options & ONI_UTF8 ? REGCODE_UTF8 : REGCODE_ASCII,
		  oni_syn[re->options & ONI_SYNTAX], &err_info);
  if (ret != REG_NORMAL) {
    static char err_buf[REG_MAX_ERROR_MESSAGE_LEN];
    re->errstr = err_buf;
    regex_error_code_to_str(&err_buf, ret, err_info);
    return 1;
  }
  re->code = code;
  return 0;
}

static int re_exec(RE *re, char *string) {
  int ret = regex_match(re->code, string, string + strlen(string),
			 string, NULL, 0);
  return ret < 0;
}

static void re_close(RE *re) {
  regex_free(re->code);
}

RE *re_oni_choise(char **s) {
  unsigned int code = 0;
  unsigned int syntax = 0;
  int end = 0;
  char *src = *s;
  static RE re_oni = {
    re_compile,
    re_exec,
    re_close,
    0,
  };
  while(!end) {
    switch(*src) {
    case 'e': syntax = ONI_POSIX_EXTENDED; src++; break;
    case 'b': syntax = ONI_POSIX_BASIC; src++; break;
    case 'E': syntax = ONI_EMACS; src++; break;
    case 'g': syntax = ONI_GREP; src++; break;
    case 'G': syntax = ONI_GNU_REGEX; src++; break;
    case 'j': syntax = ONI_JAVA; src++; break;
    case 'p': syntax = ONI_PERL; src++; break;
    case 'r': syntax = ONI_RUBY; src++; break;
    case 'u': code = ONI_UTF8; src++; break;
    default: end = 1; break;
    }
  }
  *s = src;
  re_oni.options = syntax & code;
  return &re_oni;
}

#endif
