/*
   Copyright 2000, 2001, 2002, 2003 Laurent Wacrenier

   This file is part of PLL lmtpd.

   PLL lmtpd is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   PLL lmtpd is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with PLL lmtpd; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

#include "config.h"

static char const rcsid[] UNUSED = 
"$Id: re-rfc822.c,v 1.8 2005/04/26 09:42:56 lwa Exp $";



#include <stdlib.h>
#include <string.h>
#include "re.h"
#include "../rfc/rfc.h"

char * rfc822_getaddr(char **list);

static int compar_addr(const void *a, const void *b) {
  return strcmp(*(char**)a, *(char**)b);
}

static int case_compar_addr(const void *a, const void *b) {
  return strcasecmp(*(char**)a, *(char**)b);
}

struct rfc822_tbl{ 
  int num;
  int has_domain;
  int (*compar)(const void *, const void *);
  char *addr[0];
};

static void re_close(RE *re) {
  struct rfc822_tbl *t = re->code;
  if (t) {
    int i;
    for (i=0; i<t->num; i++)
      free(t->addr[i]);
    free(t);
  }
  re->code = NULL;
}

#include <stdio.h>
static int re_compile(RE *re, char *pattern, int icase) {
  struct rfc822_tbl *t = NULL;
  int num = 0;
  char *addr ;
  int has_domain = 0;
  while (*pattern && (addr = rfc822_getaddr(&pattern)) != NULL) {
    struct rfc822_tbl *ot = t;
    num ++;
    t = realloc(t, sizeof(struct rfc822_tbl) + sizeof(char *) * num);
    if (t == NULL) {
      re->code = ot;
      re_close(re);
      re->err = RE_ERR_MEMORY;
      re->errstr = "memory";
    }
    t->addr[num-1] = addr;
    t->num = num;
    if (*addr == '@') has_domain = 1;
  }
  if (t) {
    t->has_domain = has_domain;
    if (icase)
      t->compar = compar_addr;
    else 
      t->compar = case_compar_addr;
    qsort(t->addr, t->num, sizeof(char *), t->compar);
  }
  re->code = t;
  re->err = 0;
  re->errstr = NULL;
  return 0;
}


static int re_exec(RE *re, char *string) {
  struct rfc822_tbl *tbl = (struct rfc822_tbl *)re->code;
  char *addr;
  if (tbl == NULL)
    return 1;
  while ((addr = rfc822_getaddr(&string)) != NULL) {
    if (bsearch(&addr, tbl->addr, tbl->num, sizeof(char*), tbl->compar))
      return 0;

    if (tbl->has_domain) {
      char *domain = strchr(addr, '@');
      if (domain && domain[1] && 
	  bsearch(&domain, tbl->addr, tbl->num, sizeof(char*), tbl->compar))
	  return 0;
    }
  }
  return 1;
}

RE re_rfc822 = {
  re_compile,
  re_exec,
  re_close,
  0,
  0,
};

