/*
   Copyright 2004 Laurent Wacrenier

   This file is part of PLL lmtpd.

   PLL lmtpd is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   PLL lmtpd is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with PLL lmtpd; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/


#include "config.h"

#if WITH_TRE

static char const rcsid[] UNUSED = 
"$Id: re-tre.c,v 1.5 2010/02/01 20:27:47 jcpp Exp $";

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <dlfcn.h>
#include <tre/tre.h>

#include "re.h"

const char *re_tre_init(void) {
  return NULL;
}

static void re_close(RE *re) {
  if (re->code) {
    tre_regfree(re->code);
    free(re->code);
    re->code = NULL;
  }
  if (re->hint) {
    free(re->hint);
    re->hint = NULL;
  }
}

static int re_compile(RE *re, char *pattern, int icase) {
  int ret;
  re->code = malloc(sizeof (regex_t));
  if (re->code==NULL) {
    re->err = RE_ERR_MEMORY;
    re->errstr = "memory";
  } else {
    ret = tre_regcomp(re->code, pattern, re->options | (icase?REG_ICASE:0));
    if (ret != 0) {
      if (ret == REG_ESPACE) {
        re->err =  RE_ERR_MEMORY;
      } else {
        re->err =  RE_ERR_SYNTAX;
      }
      re->errstr = "some error";
    } else {
      re->errstr = NULL;
      re->err = 0;
    }
  }
  return re->err;
}


static int re_exec(RE *re, char *string) {
  regaparams_t regparams ;
  regamatch_t regamatch = {0,};
  int ret;

  memcpy(&regparams, re->hint, sizeof(regaparams_t));

#if 0
  regparams.cost_ins=1;
  regparams.cost_del=1;
  regparams.cost_subst=1;
  regparams.max_cost=1;
  regparams.max_ins=1;
  regparams.max_del=1;
  regparams.max_subst=1;
  regparams.max_err=1;
#endif

  ret = tre_regaexec(re->code, string, &regamatch, regparams, 0);
  return ret!=0;
}

char *re_tre_params(RE *re, char *s) {
  regaparams_t *params = malloc(sizeof(regaparams_t));
  int *counter;
  int end = 0;
  tre_regaparams_default(params);
  counter = &params->max_err;
  *counter = 1;
  printf("s=%s\n", s);
  while(*s && !end) {
    if (*s>='0' && *s<='9') {
      *counter = *s - '0';
    } else switch(*s) {
    case 'i': counter = &params->max_ins; break;
    case 'd': counter = &params->max_del; break;
    case 's': counter = &params->max_subst; break;
    default: end = 1; break;
    }
    s++;
  }
  re->hint = params;
  return s;
}

RE re_tre_extended = {
  re_compile,
  re_exec,
  re_close,
  REG_EXTENDED|REG_NOSUB,
  0,
};
#endif
