/*
   Copyright 2005 Laurent Wacrenier

   This file is part of PLL lmtpd.

   PLL lmtpd is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   PLL lmtpd is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with PLL lmtpd; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

#include <stdio.h>
#include <string.h>

#include "sieve.h"

struct hdr {
  char *name;
  char *value;
};

struct hdr msg[] = 
  {
    {"From", "<lwa@teaser.fr>"},
    {"Subject", "[CIRCLE] packeteer"},
    {"To", "postmaster@teaser.fr, lwa@teaser.fr" },
    {"X-BeenThere", "secondyear-digest@list.xxx.edu"},
    {"X-Spam", ""},
    {"Number", "42"},
    { NULL }
  };

int my_valid_folder(void *message, const char *folder) {
  if (strchr(folder, '/'))
    return 0;
  return 1;
}

char *getheader(void *message, const char *header, 
		int decoded, void **state) {
  struct hdr *m = state && *state ? *state : message ;
  if (m == NULL)
    return NULL;
  while (m->name) {
    if (strcasecmp(header, m->name) == 0) {
      *state = m+1;
      return m->value;
    }
    m++;
  }
  return NULL;
}

char *from(void *message) {
  return "lwa@teaser.fr";
}

char *to(void *message) {
  return "postmaster@teaser.fr";
}

size_t size(void *message) {
  return 1000;
}

int main(int argc, char **argv) {
  struct sieve_methods Pmethods = {0,};
  struct sieve_result Presult = {0,};
  char **l;
  unsigned long n = 0;


  Pmethods.valid_folder = my_valid_folder;
  Pmethods.header = getheader;
  Pmethods.separator = '+';
  Pmethods.from = from;
  Pmethods.to = to;
  Pmethods.size = size;

#if 0
 {
   char *check[] = { "From", "To", NULL };
   void *state = NULL;
   char *hdr;
   while((hdr = getheader(msg, check, 1, &state)) != NULL) {
     printf("header: %s\n", hdr);
  }
   exit(0);
 }
#endif


 do {
   sieve((sieve_fgetc_t)fgetc, stdin, &Pmethods, msg, &Presult);
   if (Presult.implicit_keep) {
     printf("implicit keep\n");
   }
   l = Presult.redirect;
   if (l)
     while(*l) {
       printf("redirect %s\n", *l);
       l++;
     }
   l = Presult.fileinto;
   if (l) {
     while(*l) {
       printf("fileinto %s\n", *l);
      l++;
     }
   }
   if (Presult.reject) {
    printf("reject %s\n", Presult.reject);
   }
   if (Presult.error[0]) {
     printf("error at line %d: %s\n", Presult.lastline, Presult.error);
   }
   sieve_result_free(&Presult);
   rewind(stdin);
 } while (n-->0);
 return 0;
}


