/*
   Copyright 2000, 2001, 2002, 2003 Laurent Wacrenier

   This file is part of PLL lmtpd.

   PLL lmtpd is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   PLL lmtpd is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with PLL lmtpd; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

#include "config.h"

static char const rcsid[] UNUSED =
"$Id: state.c,v 1.54 2005/07/28 08:18:00 lwa Exp $";



#include <sys/types.h>
#include <netinet/in.h>
#include <netdb.h>
#include <sys/mman.h>

#include <stdlib.h>
#include <syslog.h>

#include "lmtp.h"

#ifdef HAVE_LIBHOME
#include <hpwd.h>
#else
#include <pwd.h>
#endif


void freercpt(struct rcpt *rcptlist, int nrcpt) {
  int i;
  if (nrcpt!=0) {
    for(i=0; i<nrcpt; i++)
      freercpt1(rcptlist+i);
    free(rcptlist);
  }
  return;
}

int closestate(struct fstate *state) {
  syslog(LOG_INFO, "exiting (closing connection with %s)", state->herhostname);
#if WITH_MYSQLLOG
  mysqlcloselog(state);
#endif
  quotadb_close(state);
  endpwent();
  endhostent();
  return resetstate(state, STATE_INIT);
}

/*
 * remet le status dans  l'tat "new" (vide);
 */

int resetstate(struct fstate *state, int new) {
  switch(new) {
  case STATE_INIT:
#if WITH_DB
    state->quota_db=NULL;
#endif
#if WITH_MYSQLLOG
    state->mysqllog_opened=0;
#endif
    state->counter=0;
    state->rbl_server.compar = slist_casecmp;
    state->rhsbl_server.compar = slist_casecmp;
    state->syslog_acl.compar = slist_casecmp;
    state->mod_cache.compar = slist_cmp;
    state->syslog_src = INADDR_ANY;
    state->sasl_users.compar = slist_cmp;
    state->spamassassin_acl.compar = slist_casecmp;
    state->pipe_path.compar = slist_casecmp;
    state->av.action.compar = slist_casecmp;
    state->filter_debug_msg.compar = slist_casecmp;
    state->quota_timefatal_acl.compar = slist_casecmp;

  case STATE_CHAT:
    state->counter++;

  case STATE_MAIL:
    if (state->sender) {
      free(state->sender);
      state->sender=NULL;
    }
    state->can_bounce = 0;

  case STATE_RCPT:
    if (state->rcpts) {
      freercpt(state->rcpts, state->nrcpts);
      state->rcpts=NULL;
      state->nrcpts=0;
    }
    state->nrcpts_r=0;

  case STATE_DATA:
    if (state->spool) {
      fclose(state->spool);
      state->spool=NULL;
    }
    if (state->body) {
      munmap(state->body, state->bodysize);
      state->body = NULL;
    }
    while(state->hdrlist) {
      struct headers *next;
      free(state->hdrlist->header);
      if (state->hdrlist->utf8 && 
	  state->hdrlist->utf8 != state->hdrlist->header)
	free(state->hdrlist->utf8);
      next=state->hdrlist->next;
      free(state->hdrlist);
      state->hdrlist=next;
    }
    state->hdr_in_received = NULL;
    state->hdr_delivered=NULL;
    state->msgsize=0;
    state->bodysize=0;
    state->nblines=0;
    state->nbheaders=0;
    state->messageid=NULL;
#if 0
    free_mod_cache(state->module_cache, state->module_cache_size);
    state->module_cache=NULL;
    state->module_cache_size=0;
#endif
    slist_reset(&state->rbl_server);
    slist_reset(&state->rhsbl_server);
    slist_reset(&state->mod_cache);
    av_reset(state);

  case STATE_DELIVER:
    if(state->qrecipient) {
      free(state->qrecipient);
      state->qrecipient=NULL;
    }
    if (state->hdrlist) {
      struct headers **h = &(state->hdrlist);
      while(*h) {
	if ((*h)->type >= HEADER_PRIVATE) {
	  struct headers *next = (*h)->next;
	  free((*h)->header);
	  free(*h);
	  *h=next;
	  state->nbheaders--;
	} else {
	  h=&(*h)->next;
	}
      }
    }
    if (state->syslog_host) {
      free(state->syslog_host);
      state->syslog_host = NULL;
    }
    state->user_bounce_message=NULL;
    state->rcpt=NULL;
    if (state->quotainfo) {
      mdq_close(state->quotainfo);
      state->quotainfo = NULL;
    }
    { /* reset traced headers */
      struct headers *hl = state->hdrlist;
      while(hl) {
	if (hl->trace & TRACE_DEFAULT) 
	  hl->trace |= TRACE_ON;
	else 
	  hl->trace &= ~TRACE_ON;
	hl = hl->next;
      }
    }
    state->forwards = 0;
    state->spamassassin_done = 0;
    state->spamassassin_is_spam = 0;
    state->spamassassin_score = 0;
    free_action(state);

  case STATE_FILTER:
    state->hlevel = 0;
    state->score=0;
    state->tag[0]=0;
    state->action_log = ACTION_LOG_ALL;
    if (!state->fatal)
      state->error[0]=0;
    if (state->vacation) {
      free(state->vacation);
      state->vacation=NULL;
    }
    if (state->trace) {
      free(state->trace);
      state->trace=NULL;
    }
    if (state->timezone_user) {
      free(state->timezone_user);
      state->timezone_user=NULL;
    }
    if (state->default_dest) {
      free(state->default_dest);
      state->default_dest = NULL;
    }
    if (state->mod_message) {
      free(state->mod_message);
      state->mod_message=NULL;
    }
    if (state->filter_debug) {
      fclose(state->filter_debug);
      state->filter_debug = NULL;
    }
    state->in_module = 0;
    state->logged_message = 0;

  default:
    break;
  }
  return state->state=new;
}
